import numpy as np
import matplotlib
from matplotlib import pyplot as plt
import typing

class Picker():
    def __init__(self, line: matplotlib.lines.Line2D,
                 fig: matplotlib.figure.Figure,
                 ax: matplotlib.axes.Axes):
        self.selected = []
        self.count = 0
        self.line = line
        self.growth_line = None
        self.fig = fig
        self.ax = ax
        self.xs = line.get_xdata()
        self.ys = line.get_ydata()

    def pick(self, event: matplotlib.backend_bases.PickEvent):
        if event.artist != self.line:
            return True

        N = len(event.ind)
        if not N:
            return True
        
        # mouse click location
        x = event.mouseevent.xdata
        y = event.mouseevent.ydata

        distances = np.hypot(x - self.xs[event.ind], y - self.ys[event.ind])
        indmin = distances.argmin()
        dataind = event.ind[indmin]

        self.lastind = dataind
        self.update()

    def update(self):
        if self.lastind is None:
            return
        dataind = self.lastind
        if len(self.selected) >= 2:
            self.growth_line.set_visible(False)
            self.growth_line = None
            self.ax.set_title("")
            while len(self.selected) > 0:
                p = self.selected.pop()
                p.set_visible(False)
                del p

        crt_p, = self.ax.plot(self.xs[dataind], self.ys[dataind], 'ro', ms=10, color='red', fillstyle='none')
        
        self.selected.append(crt_p)
        if len(self.selected) == 2:
            self.calc_slope()
        self.fig.tight_layout()
        self.fig.canvas.draw()

    def calc_slope(self):
        p1_x = self.selected[0].get_xdata()[0]
        p1_y = self.selected[0].get_ydata()[0]
        p2_x = self.selected[1].get_xdata()[0]
        p2_y = self.selected[1].get_ydata()[0]
        self.growth_line, = self.ax.plot([p1_x,p2_x], [p1_y,p2_y], 'r--')
        if p1_x == p2_x:
            self.ax.set_title("")
            return
        slope = np.abs((p1_y-p2_y)/(p1_x - p2_x))
        self.ax.set_title("$\gamma=$ {:.3e}".format(slope))
        
if __name__ == "__main__":
    import tkinter as tk
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk


    root = tk.Tk()
    fig = plt.figure()
    ax3 = fig.add_subplot(222)

    xs = np.linspace(0,np.pi*5,100)
    ys = np.exp(xs)#np.sin(xs) + xs

    log_line, = ax3.plot(xs, np.log(ys), picker=5)
    pic = Picker(log_line, fig, ax3)
    

    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=root)
    canvas.mpl_connect('pick_event', pic.pick)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, root)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top",fill='both',expand=True)

    root.mainloop()